@@ -651,8 +651,10 @@ class T:
     def get_parent_info(self, archive, article):
         parentID = None
         if article.in_reply_to:
-            parentID = article.in_reply_to
-        elif article.references:
+            if self.database.hasArticle(archive, article.in_reply_to):
+                # Only use In-Reply-To if it's in the archive.
+                parentID = article.in_reply_to
+        if not parentID and article.references:
             refs = self._remove_external_references(article.references)
             if refs:
                 maxdate = self.database.getArticle(archive, refs[0])
@@ -661,11 +663,15 @@ class T:
                     if a.date > maxdate.date:
                         maxdate = a
                 parentID = maxdate.msgid
-        else:
+        if not parentID:
             # Get the oldest article with a matching subject, and
             # assume this is a follow-up to that article
-            parentID = self.database.getOldestArticle(archive,
-                                                      article.subject)
+            # But, use the subject that's in the database
+            if article.decoded.has_key('stripped'):
+                subject = article.decoded['stripped'].lower()
+            else:
+                subject = article.subject.lower()
+            parentID = self.database.getOldestArticle(archive, subject)
 
         if parentID and not self.database.hasArticle(archive, parentID):
             parentID = None
