@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2006 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2013 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,10 +19,13 @@
 
 import re
 
+from types import IntType
+
 from Mailman import mm_cfg
 from Mailman import Utils
 from Mailman import Errors
 from Mailman.i18n import _
+from Mailman.htmlformat import Document
 from Mailman.Gui.GUIBase import GUIBase
 
 OPTIONS = ('hide', 'ack', 'notmetoo', 'nodupes')
@@ -150,7 +153,25 @@ class General(GUIBase):
              directive. eg.; [listname %%d] -> [listname 123]
                             (listname %%05d) -> (listname 00123)
              """)),
+          ]
 
+        if mm_cfg.ALLOW_FROM_IS_LIST:
+            rtn.append(
+                ('from_is_list', mm_cfg.Radio,
+                 (_('No'), _('Mung From'), _('Wrap Message')), 0,
+                 _("""Replace the sender with the list address to conform with
+                 policies like ADSP and DMARC.  It replaces the poster's
+                 address in the From: header with the list address and adds the
+                 poster to the Reply-To: header, but the anonymous_list and
+                 Reply-To: header munging settings below take priority.  If
+                 setting this to Yes, it is advised to set the MTA to DKIM sign
+                 all emails.""") +
+                 _("""<br>If this is set to Wrap Message, just wrap the message
+                 in an outer message From: the list with Content-Type:
+                 message/rfc822."""))
+              )
+
+        rtn.extend([
             ('anonymous_list', mm_cfg.Radio, (_('No'), _('Yes')), 0,
              _("""Hide the sender of a message, replacing it with the list
              address (Removes From, Sender and Reply-To fields)""")),
@@ -342,7 +363,7 @@ class General(GUIBase):
              type="hidden" name="new_member_options" value="ignore">'''),
 
              _("""When a new member is subscribed to this list, their initial
-             set of options is taken from the this variable's setting.""")),
+             set of options is taken from this variable's setting.""")),
 
             ('administrivia', mm_cfg.Radio, (_('No'), _('Yes')), 0,
              _('''(Administrivia filter) Check postings and intercept ones
@@ -358,6 +379,10 @@ class General(GUIBase):
              _('''Maximum length in kilobytes (KB) of a message body.  Use 0
              for no limit.''')),
 
+            ('admin_member_chunksize', mm_cfg.Number, 7, 0,
+             _('''Maximum number of members to show on one page of the
+             Membership List.''')),
+
             ('host_name', mm_cfg.Host, WIDTH, 0,
              _('Host name this list prefers for email.'),
 
@@ -367,7 +392,7 @@ class General(GUIBase):
              useful for selecting among alternative names of a host that has
              multiple addresses.""")),
 
-          ]
+          ])
 
         if mm_cfg.ALLOW_RFC2369_OVERRIDES:
             rtn.append(
@@ -408,6 +433,29 @@ class General(GUIBase):
              does not affect the inclusion of the other <tt>List-*:</tt>
              headers.)"""))
             )
+        # Suppression of Sender header modification
+        if mm_cfg.ALLOW_SENDER_OVERRIDES:
+            rtn.append(
+                ('include_sender_header', mm_cfg.Radio,
+                 (_('No'), _('Yes')), 0,
+                 _("""Should the <tt>Sender</tt> header be rewritten for this
+                 mailing list to avoid stray bounces?  <em>Yes</em> is
+                 recommended."""),
+
+                 _("""<a href="http://www.faqs.org/rfcs/rfc2822.html">RFC
+                 2822</a> defines the <tt>Sender</tt> header and defines it
+                 as "the mailbox of the agent responsible for the actual
+                 transmission of the message."  Mailman replaces this header
+                 by default with the list's bounce address.
+                 
+                 <p>While it is debatable if Mailman is such an agent, setting
+                 this header helps directing bounces from some broken MTAs to
+                 the right destination.  On the other hand, some mail
+                 readers show unexpected behaviour if this header is set (like
+                 missing addresses in forwarded mails and copies sent to the
+                 bounce address on reply-to-all), so it can be disabled 
+                 here."""))
+                )
 
         # Discard held messages after this number of days
         rtn.append(
@@ -426,27 +474,46 @@ class General(GUIBase):
             changed!  It must differ from the list's name by case
             only."""))
         elif property == 'new_member_options':
-            newopts = 0
-            for opt in OPTIONS:
+            # Get current value because there are valid bits not in OPTIONS.
+            # If we're the admin CGI, we then process the bits in OPTIONS,
+            # turning them on or off as appropriate.  Otherwise we process all
+            # the bits in mm_cfg.OPTINFO so that config_list can set and reset
+            # them.
+            newopts = mlist.new_member_options
+            if isinstance(doc, Document):
+                opts = OPTIONS
+            else:
+                opts = mm_cfg.OPTINFO
+            for opt in opts:
                 bitfield = mm_cfg.OPTINFO[opt]
                 if opt in val:
                     newopts |= bitfield
+                else:
+                    newopts &= ~bitfield
             mlist.new_member_options = newopts
         elif property == 'subject_prefix':
             # Convert any html entities to Unicode
             mlist.subject_prefix = Utils.canonstr(
                 val, mlist.preferred_language)
+        elif property == 'info':
+            if val <> mlist.info:
+                if Utils.suspiciousHTML(val):
+                    doc.addError(_("""The <b>info</b> attribute you saved
+contains suspicious HTML that could potentially expose your users to cross-site
+scripting attacks.  This change has therefore been rejected.  If you still want
+to make these changes, you must have shell access to your Mailman server.
+This change can be made with bin/withlist or with bin/config_list by setting
+mlist.info.
+                        """))
+                else:
+                    mlist.info = val
+        elif property == 'admin_member_chunksize' and (val < 1
+                                          or not isinstance(val, IntType)):
+            doc.addError(_("""<b>admin_member_chunksize</b> attribute not
+            changed!  It must be an integer > 0."""))
         else:
             GUIBase._setValue(self, mlist, property, val, doc)
 
-    def _escape(self, property, value):
-        # The 'info' property allows HTML, but let's sanitize it to avoid XSS
-        # exploits.  Everything else should be fully escaped.
-        if property <> 'info':
-            return GUIBase._escape(self, property, value)
-        # Sanitize <script> and </script> tags but nothing else.  Not the best
-        # solution, but expedient.
-        return re.sub(r'(?i)<([/]?script.*?)>', r'&lt;\1&gt;', value)
 
     def _postValidate(self, mlist, doc):
         if not mlist.reply_to_address.strip() and \
