@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2004 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2013 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,8 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
 """Bounce queue runner."""
 
@@ -88,7 +89,11 @@ class BounceMixin:
     def _queue_bounces(self, listname, addrs, msg):
         today = time.localtime()[:3]
         if self._bounce_events_fp is None:
-            self._bounce_events_fp = open(self._bounce_events_file, 'a+b')
+            omask = os.umask(006)
+            try:
+                self._bounce_events_fp = open(self._bounce_events_file, 'a+b')
+            finally:
+                os.umask(omask)
         for addr in addrs:
             cPickle.dump((listname, addr, today, msg),
                          self._bounce_events_fp, 1)
@@ -173,26 +178,59 @@ class BounceRunner(Runner, BounceMixin):
         #   who the message was destined for.  That make our job easy.
         # - the message could have been originally destined for a list owner,
         #   but a list owner address itself bounced.  That's bad, and for now
-        #   we'll simply log the problem and attempt to deliver the message to
-        #   the site owner.
-        #
-        # All messages to list-owner@vdom.ain have their envelope sender set
-        # to site-owner@dom.ain (no virtual domain).  Is this a bounce for a
-        # message to a list owner, coming to the site owner?
-        if msg.get('to', '') == Utils.get_site_email(extra='owner'):
+        #   we'll simply attempt to deliver the message to the site list
+        #   owner.
+        #   Note that this means that automated bounce processing doesn't work
+        #   for the site list.  Because we can't reliably tell to what address
+        #   a non-VERP'd bounce was originally sent, we have to treat all
+        #   bounces sent to the site list as potential list owner bounces.
+        # - the list owner could have set list-bounces (or list-admin) as the
+        #   owner address.  That's really bad as it results in a loop of ever
+        #   growing unrecognized bounce messages.  We detect this based on the
+        #   fact that this message itself will be from the site bounces
+        #   address.  We then send this to the site list owner instead.
+        # Notices to list-owner have their envelope sender and From: set to
+        # the site-bounces address.  Check if this is this a bounce for a
+        # message to a list owner, coming to site-bounces, or a looping
+        # message sent directly to the -bounces address.  We have to do these
+        # cases separately, because sending to site-owner will reset the
+        # envelope sender.
+        # Is this a site list bounce?
+        if (mlist.internal_name().lower() ==
+                mm_cfg.MAILMAN_SITE_LIST.lower()):
             # Send it on to the site owners, but craft the envelope sender to
             # be the -loop detection address, so if /they/ bounce, we won't
             # get stuck in a bounce loop.
             outq.enqueue(msg, msgdata,
-                         recips=[Utils.get_site_email()],
+                         recips=mlist.owner,
                          envsender=Utils.get_site_email(extra='loop'),
+                         nodecorate=1,
                          )
+            return
+        # Is this a possible looping message sent directly to a list-bounces
+        # address other than the site list?
+        # Check From: because unix_from might be VERP'd.
+        # Also, check the From: that Message.OwnerNotification uses.
+        if (msg.get('from') ==
+                Utils.get_site_email(mlist.host_name, 'bounces')):
+            # Just send it to the sitelist-owner address.  If that bounces
+            # we'll handle it above.
+            outq.enqueue(msg, msgdata,
+                         recips=[Utils.get_site_email(extra='owner')],
+                         envsender=Utils.get_site_email(extra='loop'),
+                         nodecorate=1,
+                         )
+            return
         # List isn't doing bounce processing?
         if not mlist.bounce_processing:
             return
         # Try VERP detection first, since it's quick and easy
         addrs = verp_bounce(mlist, msg)
-        if not addrs:
+        if addrs:
+            # We have an address, but check if the message is non-fatal.
+            if BouncerAPI.ScanMessages(mlist, msg) is BouncerAPI.Stop:
+                return
+        else:
             # See if this was a probe message.
             token = verp_probe(mlist, msg)
             if token:
@@ -201,17 +239,24 @@ class BounceRunner(Runner, BounceMixin):
             # That didn't give us anything useful, so try the old fashion
             # bounce matching modules.
             addrs = BouncerAPI.ScanMessages(mlist, msg)
+            if addrs is BouncerAPI.Stop:
+                # This is a recognized, non-fatal notice. Ignore it.
+                return
         # If that still didn't return us any useful addresses, then send it on
         # or discard it.
+        addrs = filter(None, addrs)
         if not addrs:
-            syslog('bounce', 'bounce message w/no discernable addresses: %s',
-                   msg.get('message-id'))
+            syslog('bounce',
+                   '%s: bounce message w/no discernable addresses: %s',
+                   mlist.internal_name(),
+                   msg.get('message-id', 'n/a'))
             maybe_forward(mlist, msg)
             return
         # BAW: It's possible that there are None's in the list of addresses,
         # although I'm unsure how that could happen.  Possibly ScanMessages()
         # can let None's sneak through.  In any event, this will kill them.
-        addrs = filter(None, addrs)
+        # addrs = filter(None, addrs)
+        # MAS above filter moved up so we don't try to queue an empty list.
         self._queue_bounces(mlist.internal_name(), addrs, msg)
 
     _doperiodic = BounceMixin._doperiodic
@@ -307,8 +352,12 @@ For more information see:
 """),
                              subject=_('Uncaught bounce notification'),
                              tomoderators=0)
-        syslog('bounce', 'forwarding unrecognized, message-id: %s',
+        syslog('bounce',
+               '%s: forwarding unrecognized, message-id: %s',
+               mlist.internal_name(),
                msg.get('message-id', 'n/a'))
     else:
-        syslog('bounce', 'discarding unrecognized, message-id: %s',
+        syslog('bounce',
+               '%s: discarding unrecognized, message-id: %s',
+               mlist.internal_name(),
                msg.get('message-id', 'n/a'))
